package com.pretius.pi.components;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.HttpClientBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.w3c.dom.Document;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

@Component
public class XmlPentahoParser {

	public final static String STATUS_OK = "OK";

	public final static String TASK_END = "END";
	public final static String TASK_RUNNING = "RUNNING";
	public final static String TASK_ERROR = "ERROR";
	public final static String TASK_CONN_ERROR = "CONN_ERROR";

	@Value("${pentaho.url}")
	private String pentahoUrl;

	@Value("${pentaho.getStatus}")
	private String pentahoGetStatus;

	@Value("${pentaho.startJob}")
	private String pentahoStartJob;

	@Value("${pentaho.finishedStatus}")
	private String pentahoFinishedStatus;

	@Value("${pentaho.errorStatus}")
	private String pentahoErrorStatus;

	@Value("${pentaho.cronRefreshStatus}")
	private String cronRefreshStatus;


	private static Logger logger = LoggerFactory.getLogger(XmlPentahoParser.class);
	private Map<String, String> statusMap = null;

	public Document getDocumentFromUrl(String url, String description) {
		HttpClient httpClient = HttpClientBuilder.create().build();

		try {
			HttpGet httpGet = new HttpGet(url);

			HttpResponse httpResponse = httpClient.execute(httpGet);
			HttpEntity responseEntity = httpResponse.getEntity();

			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(responseEntity.getContent());
			doc.getDocumentElement().normalize();

			return doc;

		} catch (Exception e) {
			logger.error("{} error: {}", description, e.getMessage());
		}
		return null;
	}

	public String getPentahoJobStatus(String jobName, String objectId) {
		createStatusMap();

		Document doc = getDocumentFromUrl(getPentahoGetStatusUrl(jobName, objectId), "Task status " + jobName);

		String pentahoStatusDesc = getFirstElementByTagName(doc, "status_desc");
		String taskStatus = statusMap.get(pentahoStatusDesc);
		String stackTrace = getFirstElementByTagName(doc, "log_text");

		if (TASK_ERROR.equals(taskStatus)) {

			logger.error("Job {} ended with status {}, descr: {}", jobName, taskStatus);
			logger.error("Stack trace: \n{}", stackTrace);

		} else if (!TASK_END.equals(taskStatus)) {

			logger.info("Job {} is still running with status {}", jobName, pentahoStatusDesc);
			taskStatus = TASK_RUNNING;

		} else if (taskStatus == null) {

			logger.error("Task status {} cannot get job status", jobName);
			taskStatus = TASK_CONN_ERROR;
		}

		return taskStatus;
	}

	private void createStatusMap() {
		if (statusMap == null) {
			statusMap = new HashMap<>();
			Arrays.stream(pentahoFinishedStatus.split(",")).forEach(status -> statusMap.put(status, TASK_END));
			Arrays.stream(pentahoErrorStatus.split(",")).forEach(status -> statusMap.put(status, TASK_ERROR));
		}
	}

	public String startPentahoJob(String jobName) {

		Document doc = getDocumentFromUrl(getPentahoStartJobUrl(jobName), "Starting task " + jobName);

		String status = getFirstElementByTagName(doc, "result");
		if (STATUS_OK.equals(status)) {

			String pentahoJobId = getFirstElementByTagName(doc, "id");

			return pentahoJobId;
		} else if (status != null) {
			logger.error("Pentaho start job {} failed, status: {}, message: {}", jobName, status, getErrorMessage(doc));
			return null;
		}

		logger.error("Pentaho start job {} connection error, message: {}", jobName, getErrorMessage(doc));

		return null;
	}

	private String getFirstElementByTagName(Document doc, String tagName) {
		if (doc != null && doc.getElementsByTagName(tagName).getLength() > 0) {
			return doc.getElementsByTagName(tagName).item(0).getFirstChild().getNodeValue();
		}
		return null;
	}

	private String getErrorMessage(Document doc) {
		if (doc != null && doc.getDocumentElement() != null) {
			return doc.getDocumentElement().toString();
		}
		return null;
	}

	private String getPentahoStartJobUrl(String jobName) {
		return pentahoUrl + pentahoStartJob + jobName;
	}

	private String getPentahoGetStatusUrl(String jobName, String objectId) {
		String url = pentahoUrl + pentahoGetStatus + jobName;

		if (objectId != null) {
			url += "&id=" + objectId;
		}

		return url;
	}

}
